/*---------------------------------------------------------------------------
**
**	MAIN_PLX.C
**
**
**	Copyright(c) 1999-2000, Vital Computer Solutions, All Rights Reserved
**	Richard A. Reynolds
**
**	Copyright (c) 2000-2005, ProSoft Technology, All Rights Reserved
**	Ken D. Hopwood
**
**	Copyright (c) 2002-2003, ProSoft Technology, All Rights Reserved
**	Sukhum Amorntrakul (Joe)
**
**
**	MODIFIED :
**
**		11/16/2000 - 02/26/2001 -- Created example code from
**									existing works.
**
**	Title:	This file contains the main application example code
**			for the Prolinx ADM module.
**
**  Abstract:
**
**
**	This program may serve as a starting point or an example
**	for custom applications.
**
**  Environment:		Prolinx
**                  	MS-DOS
**                  	Borland C/C++ Compiler (16-bit)
**
**
** ProSoft Technology, Inc. grants you a non-exclusive license to use, modify
** and re-distribute this program provided that this copyright notice and
** license appear on all copies of the software.
**
** Software is provided "AS IS," without a warranty of any kind. ALL EXPRESS OR
** IMPLIED REPRESENTATIONS AND WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF
** MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE
** HEREBY EXCLUDED. THE ENTIRE RISK ARISING OUT OF USING THE SOFTWARE IS ASSUMED
** BY THE LICENSEE.
**
** PROSOFT TECHNOLOGY, INC. AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY
** DAMAGES SUFFERED BY LICENSEE OR ANY THIRD PARTY AS A RESULT OF USING OR
** DISTRIBUTING SOFTWARE.  IN NO EVENT WILL PROSOFT TECHNOLOGY, INC. OR ITS
** LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT,
** INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER
** CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF
** OR INABILITY TO USE SOFTWARE, EVEN IF PROSOFT TECHNOLOGY, INC. HAS BEEN
** ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
**
**---------------------------------------------------------------------------
*/


/*---------------------------------------------------------------------------
**	COMPILER SWITCHES TO BUILD DIFFERENT VERSIONS OF THE SOFTWARE
**---------------------------------------------------------------------------
*/


/*---------------------------------------------------------------------------
**	Include files
**---------------------------------------------------------------------------
*/

#include "main_plx.h"		/* primary application header file */

#include <stdio.h>
#include <string.h>
#include <dos.h>
#include <stdlib.h>

/*---------------------------------------------------------------------------
**	CONSTANT DEFINITIONS
**---------------------------------------------------------------------------
*/


/*---------------------------------------------------------------------------
**	Function prototypes
**---------------------------------------------------------------------------
*/

static void interrupt intctl_c(void);
static void interrupt intcrit(void);

int startup(void);
int shutdown(void);
void App_LED(void);
void ShowVersion(void);
void SetLedFault(void);
void SetLedGood(void);
void SetLedStartup(void);
void SetLedShutdown(void);

extern void CommRun(int);
extern int DebugMenu(void);
extern void ProcessCfg(void);
extern int ReadCfg(void);


/*---------------------------------------------------------------------------
**	Variable definitions
**---------------------------------------------------------------------------
*/

/* structures */
ADM_INTERFACE     interface;
ADM_PRODUCT       prd;
ADM_BT_DATA       bt_data;
ADM_BLK_ERRORS    bt_err;
ADM_MODULE        module;
ADM_PORT          ports[MAX_APP_PORTS];
ADM_PORT_ERRORS   port_err[MAX_APP_PORTS];
ADMVERSIONINFO    adm_version;

/* pointers to structures */
ADM_INTERFACE * interface_ptr;

/* timer variables */
unsigned short tmlast;
long tmled;

/* status counters */
int stat_cnt = 0;

ADMHANDLE adm_handle;					   /* handle to ADM API */

static MVISPVERSIONINFO		spverinfo;	/* serial API version struct */

//baud settings for upper ports
int adjusted_baud[] =
{
   BAUD_4800,    	//BAUD110		0	/* invalid baud */
   BAUD_4800,    	//BAUD150		1	/* invalid baud */
   BAUD_150,     	//BAUD300		2	/* 300 baud */
   BAUD_300,     	//BAUD600		3	/* 600 baud */
   BAUD_600,     	//BAUD1200		4	/* 1200	baud */
   BAUD_1200,    	//BAUD2400		5	/* 2400	baud */
   BAUD_2400,    	//BAUD4800		6	/* 4800	baud */
   BAUD_4800,    	//BAUD9600		7	/* 9600	baud */
   BAUD_9600,    	//BAUD19200		8	/* 19200 baud */
	BAUD_4800,		//BAUD28800		9 	/* 28800 baud */
   BAUD_19200,   	//BAUD38400		10	/* 38400 baud */
   BAUD_28800,   	//BAUD57600		11	/* 57600 baud */
   BAUD_57600,   	//BAUD115200	12	/* 115.2 kbaud */
};

/*---------------------------------------------------------------------------
**	Functions
**---------------------------------------------------------------------------
*/

/*---------------------------------------------------------------------------
**	Ctrl-C signal handling function.  When a Ctrl-C is recognized, this
**	function is called.  The function ignores the request and must reinstall
**	the signal handler for future calls.
**---------------------------------------------------------------------------
*/

static void interrupt intctl_c(void)
{
	return;
}


static void interrupt intcrit(void)
{
	_AX = 0;						/* ignore critical error */
}


/*---------------------------------------------------------------------------
**	Main entry point
**---------------------------------------------------------------------------
*/

/*---------------------------------------------------------------------------
**
**	Entry point:
**		main
**
**	Description:
**		This is the entry point of the ADM example application.
**		This routine initializes the module, then enters the main loop
**		where the module communicates with the PLC via the backplane, and
**		communicates through the application serial ports.
**
**	Arguments:
**		none
**
**	External effects:
**
**
**	Return value:
**		none
**
**---------------------------------------------------------------------------
**	Notes:
**
**		debugging/error printf's will only be seen if console is enabled.
**
**---------------------------------------------------------------------------
*/

void main(void)
{
	int done = FALSE;
	int i;
	int rval;

	/* enable console on COM1 */
	ADM_SetConsolePort(COM1);
	/* set console to 57600 baud */
	ADM_SetConsoleSpeed(COM1, 57600L);

	/* open ADM API and get handle */
	if(ADM_Open(&adm_handle) != ADM_SUCCESS)
	{
		printf("\nFailed to open ADM API... exiting program\n");
		exit(1);
	}

	/* send status message to the status/debug port */
	printf("\nProgram Initialization....\n");

	/******************************************************************
		This section is required for the ADM API to function.
		The names of the structures may be changed to suit the
		developer's needs.  The interface structure must be initialized
		before it can be used.
	*******************************************************************
	*/

	/* initialize structure pointers */
	interface_ptr = &interface;
	interface.adm_product_ptr = &prd;
	interface.adm_bt_data_ptr = &bt_data;
	interface.adm_bt_err_ptr = &bt_err;
	interface.adm_bt_data_ptr = NULL;
	interface.adm_bt_err_ptr = NULL;
	interface.adm_module_ptr = &module;

	for(i=0;i<MAX_APP_PORTS;i++)
	{
		interface.adm_port_ptr[i] = &ports[i];
	}
	for(i=0;i<MAX_APP_PORTS;i++)
	{
		interface.adm_port_errors_ptr[i] = &port_err[i];
	}

	/* initialize ADM_INTERFACE function pointer members */
	/******************************************************

		These six functions are required if the ADM API is
		used.  The names of the functions can be changed as
		well as the code in the functions.  The interface
		structure member names must be the same however.  The
		minimum required is an empty function.  For example:

		void startup(void)
		{
		}

	*******************************************************/

	interface.startup_ptr = startup;
	interface.shutdown_ptr = shutdown;
	interface.version_ptr = ShowVersion;
	interface.user_menu_ptr = DebugMenu;
	interface.process_cfg_ptr = NULL;
	interface.ctrl_data_block_ptr = NULL;

	/* clear structures */
	memset(interface.adm_product_ptr, 0, sizeof(ADM_PRODUCT));
//	memset(interface.adm_bt_data_ptr, 0, sizeof(ADM_BT_DATA));
//	memset(interface.adm_bt_err_ptr, 0, sizeof(ADM_BLK_ERRORS));
	memset(interface.adm_module_ptr, 0, sizeof(ADM_MODULE));

	for(i=0;i<MAX_APP_PORTS;i++)
	{
		memset(interface.adm_port_ptr[i], 0, sizeof(ADM_PORT));
	}
	for(i=0;i<MAX_APP_PORTS;i++)
	{
		memset(interface.adm_port_errors_ptr[i], 0, sizeof(ADM_PORT_ERRORS));
	}

	/* clear application error flag */
	interface.Apperr = 0;

	/************************ end of section ******************************/

	/* initialize database */
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_STATUS, ADM_LED_FAULT);
	printf("Allocate & Initalize Database....\n");
	if(ADM_DBOpen(adm_handle, ADM_MAX_DB_REGS) != ADM_SUCCESS)
	{
		printf("Error setting up Database!\n");
		SetLedFault();
		return;
	}

	ADM_DBZero(adm_handle);			/* initialize database to zero */

	/* fill product structure with version data */
	strcpy(prd.ProdName,"PADM");	/* product name */
	strcpy(prd.Rev,"1.00");			/* product revision */
	strcpy(prd.Op,"0904");			/* month / year */
	strcpy(prd.Run,"2701");			/* day / run */

	disable();						   /* disable interrupts */
	setvect(0x24, intcrit);			/* install in ISR handler for CTRL-C */
	setvect(0x23, intctl_c);   	/* install in ISR handler for CTRL-C */
	setvect(0x1B, intctl_c);   	/* install in ISR handler for CTRL-Break */
	setcbrk(0);                	/* force DOS to ignore Ctrl-C */
	enable();                   	/* enable interrupts */

	printf("Open Communication Ports....\n");

	/* open application ports */
	for(i=0;i<MAX_APP_PORTS;i++)
	{
		printf("Opening COM %d\n", COM2+i+1);
		if((rval=MVIsp_Open(COM2+i, BAUD_9600, PARITY_NONE, WORDLEN8, STOPBITS1)) != MVI_SUCCESS)
		{
			printf("ERROR: Cannot open COM%d application port!\n", i+2);
         printf("Returned error: %d\n", rval);
			SetLedFault();
			return;
		}
	}

	if(startup())
	{
		shutdown();
		printf("ERROR: Startup function failed!\n");
		SetLedFault();
		return;
	}

	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_USER1, ADM_LED_ON);
	tmled = 100000L;

	interface.pass_cnt = 0;

	/* main loop for program */
	while(!done)
	{
		interface.pass_cnt++;

		/* handle protocol on each port */
		for(i=0; i < MAX_APP_PORTS; i++)
		{
			CommRun(i);						/* protocol driver */
		}

		if(ADM_ProcessDebug(adm_handle, interface_ptr) < 0)	/* debug port function */
			done = TRUE;

		/* move status data to database */
		if(module.err_freq)
		{
			stat_cnt++;
			if(stat_cnt >= module.err_freq)
			{
				ADM_SetStatus(adm_handle, interface_ptr, interface.pass_cnt);
				stat_cnt = 0;
			}
		}

		/* update timers */

		if(ADM_CheckTimer(adm_handle, &tmlast, &tmled) == 1)	/* LED flash timer for errors */
		{
			tmlast = ADM_StartTimer(adm_handle);
			tmled = 100000L;
			App_LED();
		}
	}

	shutdown();
	SetLedFault();
	ADM_Close(adm_handle);					/* close ADM API */
}


/*---------------------------------------------------------------------------
**
**	Description:
**		This routine reads the module configuration from the processor
**		and re-initializes the application serial ports.
**
**	Arguments:
**		none
**
**	External effects:
**
**
**	Return value:
**		0 = success, 1 = fail
**
**---------------------------------------------------------------------------
*/

int startup(void)
{
	int i;
	long dummy = 0L;
	int baud;
	char port_set = 0;
	char eport_set = 0;

	printf("\nReloading Program Values....\n");

	SetLedStartup();

	printf("Read Configuration....\n");
	ReadCfg();

	printf("Close Internal database....\n");
	ADM_DBClose(adm_handle);

	printf("Open Internal database....\n");
	if(ADM_DBOpen(adm_handle, module.max_regs) != ADM_SUCCESS)
	{
		return 1;
	}

	printf("Zero database....\n");
	ADM_DBZero(adm_handle);			/* initialize database to zero */

	tmlast = ADM_StartTimer(adm_handle);

	for(i=0; i < MAX_APP_PORTS; i++)
	{
		MVIsp_Close(COM2+i);

		if(i)  //if 4700-SER port
		{
			baud = adjusted_baud[ports[i].baud];

			switch(ports[i].interface)
			{
				case 1:  //RS-232
					port_set |= ((i==1) ? 0x01: ((i==2) ? 0x08: 0x40));
					break;

				case 2:  //RS-485
					port_set |= ((i==1) ? 0x02: ((i==2) ? 0x10: 0x80));
					break;

				case 3:  //RS-422
					if(i < 3)
						port_set |= ((i==1) ? 0x04: 0x20);
					else
						eport_set = 0x01;
					break;
			}
		}
		else
     	{
			baud = ports[i].baud;
     	}

		if(MVIsp_Open(COM2+i, baud, ports[i].parity, ports[i].databits,
			ports[i].stopbits) != MVI_SUCCESS)
		{
			printf("ERROR: Cannot open COM%d port for application protocol!\n",i+2);
			return 1;
		}

		if(ports[i].CTS == 'Y') 		/* check for CTS */
			MVIsp_SetHandshaking(COM2+i, HSHAKE_RTSCTS);
		else
			MVIsp_SetHandshaking(COM2+i, HSHAKE_NONE);

		MVIsp_SetRTS(COM2+i, OFF);
		MVIsp_SetDTR(COM2+i, ON);

		ADM_CheckTimer(adm_handle, &ports[i].tmlast, &dummy);
		ports[i].state    = -1;
		ports[i].ComState = 0;
	}

	//set interface configuration for upper ports
	outportb(PORT_LED_PORT, eport_set);
	outportb(SER_PORT_SET, port_set);
	PortLEDState = eport_set;

	ADM_CheckTimer(adm_handle, &tmlast, &dummy);

	ShowVersion();

	/* write buffer to console */
	while(interface.buff_len)
	{
		interface.buff_len = ADM_ConPrint(adm_handle, interface_ptr);
	}

	printf("Press ? for menu help.\n\n");

	SetLedGood();

	return 0;
}


/*---------------------------------------------------------------------------
**
**	Description:
**		This routine closes the communication ports and the backplane driver.
**
**	Arguments:
**		none
**
**	External effects:
**
**
**	Return value:
**		0 = success
**
**---------------------------------------------------------------------------
*/

int shutdown(void)
{
	int i;

	printf("\nProgram Shutdown....\n");
	printf("Closing communication ports....\n");
	for(i=0; i < MAX_APP_PORTS; i++)
	{
		MVIsp_SetRTS(COM2+i, OFF);
		MVIsp_SetDTR(COM2+i, OFF);
		MVIsp_Close(COM2+i);
	}

	/* close database */
	printf("De-allocating Module Database....\n");
	ADM_DBClose(adm_handle);

	SetLedShutdown();
	printf("Program Terminated.\n\n");
	return 0;
}


/*---------------------------------------------------------------------------
**
**	Description:
**		This routine traps communication errors and flashes the error LED on
**		and off.
**
**	Arguments:
**		none
**
**	External effects:
**
**
**	Return value:
**		none
**
**---------------------------------------------------------------------------
*/

void App_LED(void)
{
	if(!interface.Apperr)
	{
		ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);
		interface.Apperr = 0;
	}
	else
		ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_ON);
}


/*---------------------------------------------------------------------------
**
**	Description:
**		This routine displays version information.  The display
**		information is written to a buffer where the processing
**		can be controlled in order to reduce interference with
**		other processes.  If called outside of the debug menu
**		the buffer must be emptied before moving on.
**
**	Arguments:
**		none
**
**	External effects:
**		Uses debug port console buffer.
**
**	Return value:
**		none
**
**---------------------------------------------------------------------------
*/

void ShowVersion(void)
{
	ADM_GetVersionInfo(adm_handle, &adm_version);
	MVIsp_GetVersionInfo(&spverinfo);
	sprintf(interface.buff,"\nVERSION INFORMATION:\n\n\
  %s\n\
  (c) 1999-2004, ProSoft Technology, Inc.\n\n\
  PRODUCT NAME CODE         : %c%c%c%c\n\
  SOFTWARE REVISION LEVEL   : %c%c%c%c\n\
  OPERATING SYSTEM REVISION : %c%c%c%c\n\
  RUN NUMBER                : %c%c%c%c\n\n\
  SERIAL API VERSION        : %d.%d\n\
  ADM API VERSION           : %d.%d\n\n",
	PRODUCT_NAME,
	prd.ProdName[0],prd.ProdName[1],prd.ProdName[2],prd.ProdName[3],
	prd.Rev[0],prd.Rev[1],prd.Rev[2],prd.Rev[3],
	prd.Op[0],prd.Op[1],prd.Op[2],prd.Op[3],
	prd.Run[0],prd.Run[1],prd.Run[2],prd.Run[3],
	spverinfo.APISeries, spverinfo.APIRevision,
	adm_version.APIRevisionMajor, adm_version.APIRevisionMinor);

	interface.buff_ch = 0;
	interface.buff_len = strlen(interface.buff);
}


void SetLedFault(void)
{
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_ON);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_ON);
}


void SetLedGood(void)
{
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);
}

void SetLedStartup(void)
{
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_ON);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);
}

void SetLedShutdown(void)
{
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PCFG, ADM_LED_OFF);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_FAULT, ADM_LED_ON);
	ADM_SetLed(adm_handle, interface_ptr, ADM_LED_PLXOK, ADM_LED_OFF);
}

